/*
Functional test code for the TWR-DEV-PERIPH board from Axiom Manufacturing.
Adapted and modified from LCD test code created by Shu-Jen Chen of DeVry
University. 

*/

#include <hidef.h>      /* common defines and macros */
#include "derivative.h"      /* derivative-specific definitions */

//globals - 4-bit initialization sequence for LCD  -  data: PC7,6,5,4  E: PC2  R/W~: PC1 RS: PC0
const unsigned char cInit_commands[20] = {0x30,0x30,0x30,0x20,0x20,0x90,0x10,0x50,0x70,0x80,0x50,0xE0,
                                          0x60,0xA0,0x00,0xE0,0x00,0x10,0x00,0x60};
const unsigned char cMessage[3][17] = 
  { {' ','T','W','R','-','D','E','V','-','P','E','R','I','P','H',' '}, 
     {'K','e','y',' ',' ',' ','T','h','e','r','m',' ',' ',' ',' ',' '},
     {' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' '}};//data in Program Flash

const unsigned char cKey_code[] = {'1','2','3','A','4','5','6','B',
                                   '7','8','9','C','*','0','#','D'};

const unsigned char cE  = 0x04;
const unsigned char cRS = 0x01;
const unsigned char cRow_off = 0x04;    // Row offset
const byte bKey_row = 2;             // Key press output on row 2
const byte bKey_col = 4;             // Key press output position
const byte bTherm_row = 2;           // Thermistor output on row 3
const byte bTherm_col = 12;          // Thermistor output position

unsigned char cValues, cKey_row, cKey_col, cKey, cRow_cnt;
byte cTemp, cTemp_hi, cTemp_lo;

int p; 

//function prototypes
void initLCD(void);
void initPorts(void);
void LCDputs(char*);
void LCDputch(char);
void cmdwrt(unsigned char);		 
void datawrt(char);
void position(int,int);		 
void delay1u(void);		
void delay100u(void);	
void delay2m(void);		
void delays(int);
char scan_keypad(void);
void read_thermistor();

/************main************/

void main()
{
  byte temp;
  
  initPorts( );	  // port initializations
  initLCD( );	          // LCD inititialization

  for (p=0;p<3;p++){
    position(p+1,0);
    LCDputs(cMessage[p]);    //passes message address
  }
//  delays(1000);              //wait a long time

  for(;;){
  
    temp = scan_keypad();       // key pressed ?
    if (temp){
      position(bKey_row,bKey_col);  // move cursor
      datawrt(temp);            // output key press
    }
    
    read_thermistor();          // get thermistor output
    
    position(bTherm_row,bTherm_col);
    datawrt(cTemp_hi);          // send thermistor reading to LCD
    datawrt(cTemp_lo);
  }
}

/*********functions*********/
void initPorts( )
{ 
  unsigned char cValue;
  DDRP = 0x01;		// AX, PP0 output
  PTP_PTP0 = 1;		// AX, LCD BL En
  DDRB   = 0x80;    //LCD CSB active low
  DDRC   = 0xFF;    // PC7-PC4 - 4-bit LCD data bus, PC2 - E, PC1 - R/W~, PC0 - RS: all outputs
  cValue = PORTB;
  PORTB  = cValue | 0x00;  // LCD CSB (PORT B7) enabled with a logic low

  PORTA = 0x01;					// init PORTA value
  DDRA = 0x0F;					// PORTA[7:4] input, [3:0] output

  // Enable ADC on G128 MCU
  ATDCTL1_SRES = 0;					// 8 bit results
	ATDCTL3_DJM = 1;					// right justified data
	ATDCTL4_PRS = 0;					// bus clk div 2, 4 clks per sample
	ATDSTAT0_SCF = 1;					// clear flag


}	 	  				

// sends initialization commands one-by-one
void initLCD( )
{
	 unsigned char i;
	 for (i=0;i<=19;i++)
	 {  
	 	 cmdwrt(cInit_commands[i]);
	 }		
}

// sends a control word to LCD bus
void cmdwrt(unsigned char cCtrlword)
{  
	 PORTC = cCtrlword;	  // output command onto LCD data pins
	 PORTC = cCtrlword + cE;	  // generate enable pulse to latch it (xxxx x100)
	 delay1u( );		  // hold it for 1us
	 PORTC = cCtrlword;		  // end enable pulse  (xxxx x000)
	 delay2m();		  // allow 2ms to latch command inside LCD
	 PORTC = 0x00;
	 delay2m();		  // allow 2ms to latch command inside LCD
}

void position(int iRow_value, int iCol_value) 
{
     int iPos_h, iPos_l, iValue;
     if(iRow_value == 1) iValue = (0x80+0x00);
     if(iRow_value == 2) iValue = (0x80+0x10);
     if(iRow_value == 3) iValue = (0x80+0x20);
     iPos_h=((iValue + iCol_value) & 0xF0);
     iPos_l=((iValue + iCol_value) & 0x0F) << 4;
     cmdwrt(iPos_h);
     cmdwrt(iPos_l);
}

//Sends a string of characters to the LCD;...  
void LCDputs(char *sptr)
{  
     while(*sptr)
     {    //...the string must end in a 0x00 (null character)
        datawrt(*sptr);   // sptr is a pointer to the characters in the string
	++sptr;
     }
}

//Sends a single character to the LCD;...   
void LCDputch(char cValues)
{  
    while(cValues)
    {    
        datawrt(cValues);   // single character value
    }
}


// sends the character passed in by caller to LCD 
void datawrt(char cAscii)
{                   
	 char cAscii_high, cAscii_low;
	 cAscii_high = (cAscii & 0xF0);
	 cAscii_low  = (cAscii & 0x0F) << 4;  // Shift left by 4 bits 
	 PORTC = cAscii_high;              // output ASCII character upper nibble onto LCD data pins
	 PORTC = cAscii_high + cRS + cE;   // generate enable pulse to latch it  (0xxx x101)
	 delay1u( );  	// hold it for 1us
	 PORTC = cAscii_high + cRS;         // end enable pulse   (0xxx x001)
	 delay1u( );  	// hold it for 1us
	 PORTC = cAscii_low;               // output ASCII character lower nibble onto LCD data pins
	 PORTC = cAscii_low + cRS + cE;    // generate enable pulse to latch it  (0xxx x101)
	 delay1u( );  	// hold it for 1us
	 PORTC = cAscii_low + cRS;         // end enable pulse   (0xxx x001)
	 delay100u( );                     // allow 100us to latch data inside LCD
}

void delay1u( )
{
	 unsigned int i;
	 for(i=0;i<=0x0f;i++)
	 { /* adjust condition field for delay time */
	 	 asm("nop");
	 }
}

void delay100u( )
{
	 unsigned int i,j;
	 for(i=0;i<=0x02;i++)
	 {	 /* adjust condition field for delay time */
	  	for(j=0;j<=0xff;j++)
	  	{
	 	    asm("nop");
	  	}
	 }
}

void delay2m( )
{
   unsigned int i,j;	
   for (i=0;i<=0x20;i++)
   { /* adjust condition field for delay time */
 	   for (j=0;j<=0xff;j++)
 	   {
	     asm("nop");
	   }		   
   }
}    

void delays(int k )
{
   unsigned int i,j;	
   for (i=0;i<=k;i++)
   { /* adjust condition field for delay time */
 	   for (j=0;j<=0xff;j++)
 	   {
	     asm("nop");
	   }		   
   }
}

/*  scan keypad and return ASCII value of key pressed */
char scan_keypad(){
//  byte n, m;
  
  PORTA = 0x01;           // set to col 1
  for(cKey_col=0;cKey_col<4;cKey_col++){     // scan all col's
    if(PORTA & 0xF0){                   // if any sw pressed this col
      cKey_row = (PORTA & 0xF0) >> 4;   // capture row num in low nibble
      if (cKey_row==4) cKey_row = 3;  
      if (cKey_row==8) cKey_row = 4;    // adjust row num
      cRow_cnt = (cKey_row-1) * 0x04;   // adjust row count
      return(*(cKey_code+(cRow_cnt+cKey_col)));  // return key pressed
    }
    PORTA = PORTA << 1;
  }
  return 0;
}

void read_thermistor(){

    ATDCTL5_Cx = 1;				      // start conversion on thermistor
		while(!(ATDSTAT0_SCF));		  // wait for conversion complete

		cTemp = (byte)(ATDDR1 & 0xFF);      // get converted value, cast to byte
		
		cTemp_hi = (cTemp & 0xF0) >> 4; // parse ADC output to display on LCD
		cTemp_lo = cTemp & 0x0F;
		
		if(cTemp_hi <= 0x09){          // format output to dispaly on LCD
		  cTemp_hi = cTemp_hi + 0x30;
		} else {
		  cTemp_hi = cTemp_hi +0x37;
		}
		if(cTemp_lo <= 0x09){          // format output to dispaly on LCD
		  cTemp_lo = cTemp_lo + 0x30;
		} else {
		  cTemp_lo = cTemp_lo +0x37;
		}

  
}


        /********* end of file ***********************/

